@@ -108,6 +108,11 @@ impl Object {
     /// - `touch path/to/file`
     /// - `mkdir path/to/dir/`
     ///
+    /// # Behavior
+    ///
+    /// - Create on existing dir will succeed.
+    /// - Create on existing file will overwrite and truncate it.
+    ///
     /// # Examples
     ///
     /// ## Create an empty file
@@ -250,7 +255,7 @@ impl Object {
     /// ```
     pub async fn range_reader(&self, range: impl RangeBounds<u64>) -> Result<impl BytesRead> {
         let op = OpRead::new(self.meta.path(), range)?;
-        Ok(self.acc.read(&op).await?)
+        self.acc.read(&op).await
     }
 
     /// Create a reader which implements AsyncRead and AsyncSeek inside specified range.
